[inherit('common','xmlrpc_format')]
program test_xmlrpc_format(input,output);

procedure test1;

const
    expect = '<methodCall><methodName>test1</methodName><params><param><value><i4>123</i4></value></param><param><value><i4>456</i4></value></param></params></methodCall>';

var
    actual : pstr;

begin
    pxmlrpc_request(actual, 'test1', pxmlrpc_i4(123), pxmlrpc_i4(456));
    if actual <> expect then begin
        writeln('Format i4 failure');
        writeln('  expect : ' + expect);
        writeln('  actual : ' + actual);
    end;
end;

procedure test2;

const
   expect = '<methodCall><methodName>test2</methodName><params><param><value><string>ABC</string></value></param><param><value><string>DEF</string></value></param></params></methodCall>';

var
   actual : pstr;

begin
    pxmlrpc_request(actual, 'test2', pxmlrpc_string('ABC'), pxmlrpc_string('DEF'));
    if actual <> expect then begin
        writeln('Format string failure');
        writeln('  expect : ' + expect);
        writeln('  actual : ' + actual);
    end;
end;

procedure test3;

const
   expect = '<methodCall><methodName>test3</methodName><params><param><value>' +
            '<struct><member><name>ival</name><value><i4>123</i4></value></member><member><name>sval</name><value><string>ABC</string></value></member></struct>' +
            '</value></param></params></methodCall>';

var
   actual : pstr;

begin
    pxmlrpc_request(actual, 'test3', pxmlrpc_struct2(
                                         pxmlrpc_member('ival', pxmlrpc_i4(123)),
                                         pxmlrpc_member('sval', pxmlrpc_string('ABC'))
                                     ));
    if actual <> expect then begin
        writeln('Format struct failure');
        writeln('  expect : ' + expect);
        writeln('  actual : ' + actual);
    end;
end;

procedure test4;

const
    expect = '<methodCall><methodName>test4</methodName><params><param><value>' +
             '<array><data><value><i4>123</i4></value><value><i4>456</i4></value></data></array>' +
             '</value></param></params></methodCall>';

var
    actual: pstr;

begin
    pxmlrpc_request(actual, 'test4', pxmlrpc_array(
                                         pxmlrpc_i4(123),
                                         pxmlrpc_i4(456)
                                     ));
    if actual <> expect then begin
        writeln('Format array i4 failure');
        writeln('  expect : ' + expect);
        writeln('  actual : ' + actual);
    end;
end;

procedure test5;

const
    expect = '<methodCall><methodName>test5</methodName><params><param><value>' +
             '<array><data><value><string>ABC</string></value><value><string>DEF</string></value></data></array>' +
             '</value></param></params></methodCall>';

var
    actual : pstr;

begin
    pxmlrpc_request(actual, 'test5', pxmlrpc_array(
                                         pxmlrpc_string('ABC'),
                                         pxmlrpc_string('DEF')
                                     ));
    if actual <> expect then begin
        writeln('Format array string failure');
        writeln('  expect : ' + expect);
        writeln('  actual : ' + actual);
    end;
end;

begin
   test1;
   test2;
   test3;
   test4;
   test5;
end.
