[inherit('common', 'xmlrpc_parse')]
program test_xmlrpc_parse(input, output);

procedure test1(xmlstr : pstr);

var
    v, errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        pxmlrpc_response(xmlstr, iaddress(v));
        if v <> 123 then begin
            writeln('Parse i4 failure');
            writeln('  expect : 123');
            writeln('  actual : ', v:1);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

procedure test2(xmlstr : pstr);

var
    s : c_str_t;
    s2 : pstr;
    errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        pxmlrpc_response(xmlstr, iaddress(s));
        s2 := pas_str(s);
        free_c_str(s);
        if s2 <> 'ABC' then begin
            writeln('Parse string failure');
            writeln('  expect : ABC');
            writeln('  actual : ', s2);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

procedure test3(xmlstr : pstr);

var
    ia : array [1..1000] of integer;
    n, errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr)then begin
        n := pxmlrpc_response(xmlstr, iaddress(ia));
        if n <> 2 then begin
            writeln('Parse array i4 num element failure');
            writeln('  expect : 2');
            writeln('  actual : ', n:1);
        end;
        if ia[1] <> 123 then begin
            writeln('Parse array i4 element 1 failure');
            writeln('  expect : 123');
            writeln('  actual : ', ia[1]:1);
        end;
        if ia[2] <> 456 then begin
            writeln('Parse array i4 element 2 failure');
            writeln('  expect : 456');
            writeln('  actual : ', ia[2]:1);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

procedure test4(xmlstr : pstr);

var
    sa : array [1..1000] of c_str_t;
    sa2 : array [1..1000] of pstr;
    n, i, errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        n := pxmlrpc_response(xmlstr, iaddress(sa));
        for i := 1 to n do begin
            sa2[i] := pas_str(sa[i]);
            free_c_str(sa[i]);
        end;
        if n <> 2 then begin
            writeln('Parse array string num element failure');
            writeln('  expect : 2');
            writeln('  actual : ', n:1);
        end;
        if sa2[1] <> 'ABC' then begin
            writeln('Parse array string element 1 failure');
            writeln('  expect : ABC');
            writeln('  actual : ', sa2[1]);
        end;
        if sa2[2] <> 'DEF' then begin
            writeln('Parse array string element 2 failure');
            writeln('  expect : DEF');
            writeln('  actual : ', sa2[2]);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

type
   data = record
             ival : integer;
             sval : pstr;
          end;

procedure test5(xmlstr : pstr);

var
    ctx : xmlrpc_struct_t;
    d : data;
    errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        ctx := pxmlrpc_struct_init(xmlstr, 2);
        d.ival := pxmlrpc_struct_unpack_i4(ctx, 0, 'ival');
        d.sval := pxmlrpc_struct_unpack_string(ctx, 0, 'sval');
        pxmlrpc_struct_cleanup(ctx);
        if d.ival <> 123 then begin
            writeln('Parse struct i4 failure');
            writeln('  expect : 123');
            writeln('  actual : ', d.ival:1);
        end;
        if d.sval <> 'ABC' then begin
            writeln('Parse struct string failure');
            writeln('  expect : ABC');
            writeln('  actual : ', d.sval);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

procedure test6(xmlstr : pstr);

var
    ctx : xmlrpc_struct_t;
    d : array [1..1000] of data;
    n, i : integer;
    errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        ctx := pxmlrpc_struct_init(xmlstr, 2);
        n := pxmlrpc_struct_n(ctx);
        for i := 1 to n do begin
           d[i].ival := pxmlrpc_struct_unpack_i4(ctx, i - 1, 'ival');
           d[i].sval := pxmlrpc_struct_unpack_string(ctx, i - 1, 'sval');
        end;
        pxmlrpc_struct_cleanup(ctx);
        if n <> 2 then begin
            writeln('Parse array struct num element failure');
            writeln('  expect : 2');
            writeln('  actual : ', n:1);
        end;
        if d[1].ival <> 123 then begin
            writeln('Parse array struct i4 element 1 failure');
            writeln('  expect : 123');
            writeln('  actual : ', d[1].ival:1);
        end;
        if d[1].sval <> 'ABC' then begin
            writeln('Parse array struct string element 1 failure');
            writeln('  expect : ABC');
            writeln('  actual : ', d[1].sval);
        end;
        if d[2].ival <> 456 then begin
            writeln('Parse array struct i4 element 2 failure');
            writeln('  expect : 456');
            writeln('  actual : ', d[2].ival);
        end;
        if d[2].sval <> 'DEF' then begin
            writeln('Parse array struct string element 2 failure');
            writeln('  expect : DEF');
            writeln('  actual : ', d[2].sval);
        end;
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        writeln('  error: ', errcod:1, ' ', errmsg);
    end;
end;

procedure test7(xmlstr : pstr);

var
    errcod : integer;
    errmsg : pstr;

begin
    if pxmlrpc_response_status(xmlstr) then begin
        writeln('Ooops');
    end else begin
        pxmlrpc_response_errmsg(xmlstr, errcod, errmsg);
        if errcod <> 101 then begin
            writeln('Parse error code failure');
            writeln('  expect : 101');
            writeln('  actual : ', errcod:1);
        end;
        if errmsg <> 'Expected error' then begin
            writeln('Parse error message failure');
            writeln('  expect : Expected error');
            writeln('  actual : ', errmsg);
        end;
    end;
end;

begin
    test1('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value><i4>123</i4></value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test2('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value><string>ABC</string></value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test3('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value>' + chr(10) +
          '        <array>' + chr(10) +
          '          <data>' + chr(10) +
          '            <value><i4>123</i4></value>' + chr(10) +
          '            <value><i4>456</i4></value>' + chr(10) +
          '          </data>' + chr(10) +
          '        </array>' + chr(10) +
          '      </value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test4('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value>' + chr(10) +
          '        <array>' + chr(10) +
          '          <data>' + chr(10) +
          '            <value><string>ABC</string></value>' + chr(10) +
          '            <value><string>DEF</string></value>' + chr(10) +
          '          </data>' + chr(10) +
          '        </array>' + chr(10) +
          '      </value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test5('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value>' + chr(10) +
          '        <struct>' + chr(10) +
          '          <member>' + chr(10) +
          '            <name>ival</name>' + chr(10) +
          '            <value><i4>123</i4></value>' + chr(10) +
          '          </member>' + chr(10) +
          '          <member>' + chr(10) +
          '            <name>sval</name>' + chr(10) +
          '            <value><string>ABC</string></value>' + chr(10) +
          '          </member>' + chr(10) +
          '        </struct>' + chr(10) +
          '      </value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test6('<methodResponse>' + chr(10) +
          '  <params>' + chr(10) +
          '    <param>' + chr(10) +
          '      <value>' + chr(10) +
          '        <array>' + chr(10) +
          '          <data>' + chr(10) +
          '            <struct>' + chr(10) +
          '              <member>' + chr(10) +
          '                <name>ival</name>' + chr(10) +
          '                <value><i4>123</i4></value>' + chr(10) +
          '              </member>' + chr(10) +
          '              <member>' + chr(10) +
          '                <name>sval</name>' + chr(10) +
          '                <value><string>ABC</string></value>' + chr(10) +
          '              </member>' + chr(10) +
          '            </struct>' + chr(10) +
          '            <struct>' + chr(10) +
          '              <member>' + chr(10) +
          '                <name>ival</name>' + chr(10) +
          '                <value><i4>456</i4></value>' + chr(10) +
          '              </member>' + chr(10) +
          '              <member>' + chr(10) +
          '                <name>sval</name>' + chr(10) +
          '                <value><string>DEF</string></value>' + chr(10) +
          '              </member>' + chr(10) +
          '            </struct>' + chr(10) +
          '          </data>' + chr(10) +
          '        </array>' + chr(10) +
          '      </value>' + chr(10) +
          '    </param>' + chr(10) +
          '  </params>' + chr(10) +
          '</methodResponse>');
    test7('<methodResponse>' + chr(10) +
          '  <fault>' + chr(10) +
          '    <value>' + chr(10) +
          '      <struct>' + chr(10) +
          '        <member>' + chr(10) +
          '          <name>faultCode</name>' + chr(10) +
          '          <value><int>101</int></value>' + chr(10) +
          '        </member>' + chr(10) +
          '        <member>' + chr(10) +
          '          <name>faultString</name>' + chr(10) +
          '          <value><string>Expected error</string></value>' + chr(10) +
          '        </member>' + chr(10) +
          '      </struct>' + chr(10) +
          '    </value>' + chr(10) +
          '  </fault>' + chr(10) +
          '</methodResponse>');
    test1('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value><i4>123</i4></value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test2('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value><string>ABC</string></value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test3('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<array>' +
          '<data>' +
          '<value><i4>123</i4></value>' +
          '<value><i4>456</i4></value>' +
          '</data>' +
          '</array>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test4('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<array>' +
          '<data>' +
          '<value><string>ABC</string></value>' +
          '<value><string>DEF</string></value>' +
          '</data>' +
          '</array>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test5('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>123</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value><string>ABC</string></value>' +
          '</member>' +
          '</struct>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test6('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<array>' +
          '<data>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>123</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value><string>ABC</string></value>' +
          '</member>' +
          '</struct>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>456</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value><string>DEF</string></value>' +
          '</member>' +
          '</struct>' +
          '</data>' +
          '</array>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test7('<methodResponse>' +
          '<fault>' +
          '<value>' +
          '<struct>' +
          '<member>' +
          '<name>faultCode</name>' +
          '<value><int>101</int></value>' +
          '</member>' +
          '<member>' +
          '<name>faultString</name>' +
          '<value><string>Expected error</string></value>' +
          '</member>' +
          '</struct>' +
          '</value>' +
          '</fault>' +
          '</methodResponse>');
    test2('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>ABC</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test4('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<array>' +
          '<data>' +
          '<value>ABC</value>' +
          '<value>DEF</value>' +
          '</data>' +
          '</array>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test5('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>123</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value>ABC</value>' +
          '</member>' +
          '</struct>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
    test6('<methodResponse>' +
          '<params>' +
          '<param>' +
          '<value>' +
          '<array>' +
          '<data>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>123</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value>ABC</value>' +
          '</member>' +
          '</struct>' +
          '<struct>' +
          '<member>' +
          '<name>ival</name>' +
          '<value><i4>456</i4></value>' +
          '</member>' +
          '<member>' +
          '<name>sval</name>' +
          '<value>DEF</value>' +
          '</member>' +
          '</struct>' +
          '</data>' +
          '</array>' +
          '</value>' +
          '</param>' +
          '</params>' +
          '</methodResponse>');
end.
